## List available Landsat 5 image directories.  
## For each directory, rescale band 5.
## Apply threshold to create binary water map.
## Create a mask to extract non-cloud (good) pixels
## for later use in the analysis.

## NESSF Dissertation Project
## Danica Schaffer-Smith March 17, 2015
## djs50@duke.edu

import arcpy
from arcpy import env
from arcpy.sa import *
import glob

## Set workspace and scratch directory
arcpy.env.workspace = "C:/Users/djs50/Desktop/Landsat4_5"
env.scratchWorkspace = "C:/Users/djs50/Desktop/Landsat4_5/044033/output/scratch"

# Set reference image to be used in setting extent, snap raster, cell size
refImage = arcpy.Raster("C:/Users/djs50/Desktop/Landsat4_5/mask/Sac_WRS2_CVJV_extent.tif")

## Set environments to match the reference study area raster
# arcpy.env.extent = arcpy.Extent(-107.0, 38.0, -104.0, 40.0) # This method sets dimensions
arcpy.env.extent = refImage # This method references a file
arcpy.env.snapRaster = refImage
arcpy.env.cellSize = refImage

## Loop through each scene directory
## Search for scene directory list
dirlist = glob.glob("C:/Users/djs50/Desktop/Landsat4_5/"+'04*')
print "Number of scenes: " +str(len(dirlist))
print dirlist

## Loop through each scene directory
for thisdir in dirlist:
    ## List the original cdr datasets available
    cdrlist = glob.glob(thisdir + "/raw_data/L*")
    print "Number of cdr files: " + str(len(cdrlist))
    ### Add log file?
    ###
    for cdr in cdrlist:
        ## Find band5
        b5list = glob.glob(cdr + "/*_sr_band5.img")
        print b5list
        
        ## Extract the SceneID string from band 5
        filename = b5list[0]
        print filename
        mysplit = filename.split("\\")
        lastname = mysplit[-1]
        print lastname
        mysplit2 = lastname.split("_")
        SceneID = mysplit2[0]
        
        ## Find band 5
        b5list = glob.glob(cdr + "/*_sr_band5.img")
        print b5list
        
        ## Rescale band 5 (swir)
        arcpy.CheckOutExtension('Spatial')
        arcpy.env.overwriteOutput = True
        inB5 = arcpy.Raster(b5list[0])
        outB5 = inB5 * 0.0001

        ## Find band 2
        #b2list = glob.glob(cdr + "/*_sr_band2.img")
        #print b2list
        
        ## Rescale band 2
        #inB2 = arcpy.Raster(b2list[0])
        #outB2 = inB2 * 0.0001
              
        ## Calculate the MNDWI and save the output
        #mndwi = (outB2 - outB5) / (outB2 + outB5)
        #mndwi.save(thisdir + "\\output\\mndwi\\" + SceneID + "_sr_mndwi.img")

        ## Find the preprocessed ndvi layer
        ndvilist = glob.glob(cdr + "/*_ndvi.img")
        print ndvilist
        
        ## Rescale ndvi 
        inndvi = arcpy.Raster(ndvilist[0])
        outndvi = inndvi * 0.0001
                             
        ## Apply a threshold to band 5 to create a binary water map
        ## values =< 0.0690, 1
        ## values >= 0.0690, 0            
        inRaster = outB5
        reclassField = "Value"
        remap = arcpy.sa.RemapRange([[-30,0.0690, 1], [0.0690,30, 0]])
        reclassb5 = arcpy.sa.Reclassify(inRaster, "Value", remap, "NODATA")
        reclassb5.save(thisdir + "\\output\\scratch\\threshold.tif")

        ## Find the cfmask
        cflist = glob.glob(cdr + "/*_cfmask.img")
        print cflist
        
        ## Reclassify the cf mask to create a mask of good non-cloud pixels
        ## 0 = clear
        ## 1 = water
        ## 2 = shadow
        ## 3 = snow
        ## 4 = cloud
        ## 255 = fill
        cf = arcpy.Raster(cflist[0])
        #Reclassify non-cloud pixels to 1 and cloud pixels to NODATA
        remap2 = arcpy.sa.RemapRange([[0, 1, 1],[2,2,"NODATA"], [4,4,"NODATA"], [3,3,1]]) 
        reclasscf = arcpy.sa.Reclassify(cf,"Value", remap2, "NODATA")
        reclasscf.save(thisdir + "\\output\\noncloud\\" + SceneID + "_sr_noncloud.img")
        
        ## extract non-cloud pixels from band 5
        inRas = reclassb5
        mask = reclasscf
        outExtractCf = arcpy.sa.ExtractByMask(inRas, mask)
        outExtractCf.save(thisdir + "\\output\\scratch\\water_b5.tif")
        
        ## extract band 5 by the study area (combined CVJV plan area, elevation <70m, non-urban USDA CDL)
        inRas = outExtractCf
        mask = arcpy.Raster("C:/Users/djs50/Desktop/Landsat4_5/mask/telemetry_mask_nonurban.tif")
        outExtract2 = arcpy.sa.ExtractByMask(inRas, mask)
        outExtract2.save(thisdir + "\\output\\scratch\\extract_nonurban.tif")
      
        ## Copy Raster and increase extent to match reference image
        #arcpy.env.extent = "555342.557462497 4208145 686832.557462497 4462575"
        arcpy.env.extent = refImage
        arcpy.env.snapRaster = refImage
        arcpy.CopyRaster_management(outExtract2, thisdir + "\\output\\masked_water_map\\" + SceneID + "_sr_water.img")

        ## Do the same cloud elimination procedure for the ndvi image
        ## extract non-cloud pixels from ndvi
        inRas = outndvi
        mask = reclasscf
        outExtractCf = arcpy.sa.ExtractByMask(inRas, mask)
        outExtractCf.save(thisdir + "\\output\\scratch\\ndvi_ncloud.tif")
        
        ## extract band 5 by the study area (combined CVJV plan area, elevation <70m, non-urban USDA CDL)
        inRas = outExtractCf
        mask = arcpy.Raster("C:/Users/djs50/Desktop/Landsat4_5/mask/telemetry_mask_nonurban.tif")
        outExtract2 = arcpy.sa.ExtractByMask(inRas, mask)
        outExtract2.save(thisdir + "\\output\\scratch\\extract_ndvi_nonurban.tif")
      
        ## Copy Raster and increase extent to match reference image
        arcpy.env.extent = refImage
        arcpy.env.snapRaster = refImage
        arcpy.CopyRaster_management(outExtract2, thisdir + "\\output\\masked_ndvi\\" + SceneID + "_sr_ndvi.img")

        ## Clean up temporary files
        ## arcpy.Delete_management(tmpFC)  
