# List available Landsat 7 image directories.
# For each directory, rescale band 5.
# Apply threshold to create binary water map.
# Create a mask to extract non-cloud (good) pixels
# for later use in the analysis.

# Danica Schaffer-Smith January 15, 2016
# djs50@duke.edu

import arcpy
from arcpy import env
from arcpy.sa import *
import glob

## Set workspace and scratch directory
arcpy.env.workspace = "H:/NESSF_Sac/Landsat7"
env.scratchWorkspace = "H:/NESSF_Sac/Landsat7/044033/output/scratch"

# Set reference image to be used in setting extent, snap raster, cell size
refImage = arcpy.Raster("H:/NESSF_Sac/Landsat7/mask/Sac_WRS2_CVJV_extent.tif")

## Set environments to match the reference study area raster
# arcpy.env.extent = arcpy.Extent(-107.0, 38.0, -104.0, 40.0) # This method sets dimensions
arcpy.env.extent = refImage # This method references a file
arcpy.env.snapRaster = refImage
arcpy.env.cellSize = refImage

## Loop through each scene directory
## Search for scene directory list
dirlist = glob.glob("H:/NESSF_Sac/Landsat7/"+'04*')
print "Number of scenes: " +str(len(dirlist))
print dirlist

## Loop through each scene directory
for thisdir in dirlist:
    ## List the original cdr datasets available
    cdrlist = glob.glob(thisdir + "/raw_data/L*")
    print "Number of cdr files: " + str(len(cdrlist))
    ### Add log file?
    ###
    for cdr in cdrlist:
        ## Find band5
        b5list = glob.glob(cdr + "/*_sr_band5.img")
        print b5list

        ## Extract the SceneID string from band 5
        filename = b5list[0]
        print filename
        mysplit = filename.split("\\")
        lastname = mysplit[-1]
        print lastname
        mysplit2 = lastname.split("_")
        SceneID = mysplit2[0]

        ## Rescale band 5 (swir)
        arcpy.CheckOutExtension('Spatial')
        arcpy.env.overwriteOutput = True
        inB5 = arcpy.Raster(b5list[0])
        outB5 = inB5 * 0.0001

        ## Find band 2 (green)
        #b2list = glob.glob(cdr + "/*_sr_band2.img")
        #print b2list

        ## Rescale band 2
        #arcpy.CheckOutExtension('Spatial')
        #arcpy.env.overwriteOutput = True
        #inB2 = arcpy.Raster(b2list[0])
        #outB2 = inB2 * 0.0001

        ## Compute the MNDWI
        #mndwi = ((outB2 - outB5) / (outB2 + outB5))
        #mndwi.save(thisdir + "\\output\\mndwi\\" + SceneID + "_sr_mndwi.img")

        ## Apply a threshold to create a binary water map
        ## b5 values =< 0.0690, 1
        ## b5 values >= 0.0690, 0
        #inRaster = outB7
        #reclassField = "Value"
        #remap = arcpy.sa.RemapRange([[-30,-0.07, 0], [-0.07,30, 1]])
        #reclassB7 = arcpy.sa.Reclassify(ndwi, "Value", remap, "NODATA")
        #reclassB7.save(thisdir + "\\B7_Threshold\\b7thresh_" + SceneID + ".tif")
        reclassB5 = Con(outB5 <= 0.0690, 1, 0)
        reclassB5.save(thisdir + "/output/scratch/threshold.tif")

        ## Extract by mask
        inRas = reclassB5
        mask = arcpy.Raster("H:/NESSF_Sac/Landsat7/mask/telemetry_mask_nonurban.tif") # CVJV plan area non-urban zones, not Sutter Butte
        outExtractByMask = arcpy.sa.ExtractByMask(inRas, mask)
        outExtractByMask.save(thisdir + "/output/scratch/extract1.tif")

        ## Raster to points
        # set local variables
        inRas = outExtractByMask
        outPt = thisdir + "/output/scratch/b5pts.shp"
        field = "VALUE"

        # Execute RasterToPoint conversion
        arcpy.RasterToPoint_conversion(inRas, outPt, field)

        ## Run Inverse Distance Weighting (IDW) with the land boundaries file to interpolate
        ## values within the striped areas of Landsat 7.

        # Set local variables
        inPointFeatures = outPt
        zField = "GRID_CODE"
        cellSize = 30.0
        power = 2
        searchRadius = arcpy.sa.RadiusVariable(10, 250)
        bound = ("H:/NESSF_Sac/Landsat7/Master_land_boundary_lines_UTM10N.shp") # line boundary file

        # Execute IDW
        outIDW = arcpy.sa.Idw(inPointFeatures, zField, cellSize, power, searchRadius)
        outIDW.save(thisdir + "/output/scratch/b5idw.tif")

        ## Reclassify the IDW result into a binary water/no water raster
        ## This way both rasters will be 8-bit, unsigned integers, which can be combined.
        remap = arcpy.sa.RemapRange([[0,0.5, 0], [0.5,1, 1]])
        reclassIdw = arcpy.sa.Reclassify(outIDW, "VALUE", remap, "NODATA")
        reclassIdw.save(thisdir + "/output/scratch/b5_water.tif")

        ## Find the cloud mask
        #ncloudlist = glob.glob(thisdir + "/output/nonCloud/ncloudidwre_" + SceneID + ".tif")
        #print ncloudlist

        ## Find the cfmask
        cflist = glob.glob(cdr + "/*_cfmask.img")
        print cflist

        ## Reclassify the cf mask to create a mask of good non-cloud pixels
        ## 0 = clear
        ## 1 = water
        ## 2 = shadow
        ## 3 = snow
        ## 4 = cloud
        ## 255 = fill
        cf = arcpy.Raster(cflist[0])

        #Reclassify non-cloud pixels to 1 and cloud and shadow pixels to 0
        remap2 = arcpy.sa.RemapRange([[0, 1, 1],[2,2,0], [4,4,0], [3,3,1]])
        reclasscf = arcpy.sa.Reclassify(cf,"Value", remap2, "NODATA")
        reclasscf.save(thisdir + "/output/scratch/ncloud.tif")

        ## extract values in the study area mask to clip out unimportant areas
        inRas = reclasscf
        mask = arcpy.Raster("H:/NESSF_Sac/Landsat7/mask/telemetry_mask_nonurban.tif")
        outExtractCf = arcpy.sa.ExtractByMask(inRas, mask)
        outExtractCf.save(thisdir + "/output/scratch/extract_nonurban.tif")

        ## convert the raster to points to guide an idw operation
        inRas = outExtractCf
        outPt = thisdir + "/output/scratch/ncloud_idwpoints.shp"
        field = "VALUE"
        arcpy.RasterToPoint_conversion(inRas, outPt, field)

        ## conduct IDW to fill gaps in the cloud mask
        inPointFeatures = outPt
        zField = "GRID_CODE"
        cellSize = 30.0
        power = 2
        searchRadius = arcpy.sa.RadiusVariable(10, 250)
        cfIDW = arcpy.sa.Idw(inPointFeatures, zField, cellSize, power, searchRadius)
        cfIDW.save(thisdir + "/output/scratch/ncloud_idw.tif")

        ## Reclassify the idw to make a mask where non-cloud = 1, cloud = NODATA
        remap = arcpy.sa.RemapRange([[0,0.5, "NODATA"], [0.5,1, 1]])
        ncloud = arcpy.sa.Reclassify(cfIDW, "VALUE", remap)
        ncloud.save(thisdir + "/output/noncloud/" + SceneID + "_sr_noncloud.img")

        ## Apply a cloudmask to remove clouded areas from the classification.
        inRas = reclassIdw
        mask2 = ncloud
        outExtractByMask2 = arcpy.sa.ExtractByMask(inRas, mask2)
        outExtractByMask2.save(thisdir + "/output/scratch/rmcloud.tif")

        ## Copy Raster and increase extent to match reference image
        #refImage = arcpy.Raster("H:/NESSF_Sac/Landsat7/mask/Sac_WRS2_CVJV_extent.tif")
        #arcpy.env.extent = "555342.557462497 4208145 686832.557462497 4462575"
        arcpy.env.extent = refImage
        arcpy.env.snapRaster = refImage
        arcpy.CopyRaster_management(outExtractByMask2, thisdir + "/output/masked_water_map/" + SceneID + "_sr_water.img")
