## List available Landsat 8 image directories.  
## For each directory, rescale band 8.
## Apply threshold to create binary water map.
## Create a mask to extract non-cloud (good) pixels
## for later use in the analysis.

## NESSF Dissertation Project
## Danica Schaffer-Smith January 15, 2016
## djs50@duke.edu

import arcpy
from arcpy import env
from arcpy.sa import *
import glob

## Set workspace and scratch directory
arcpy.env.workspace = "H:/NESSF_Sac/Landsat8"
env.scratchWorkspace = "H:/NESSF_Sac/Landsat8/044033/output/scratch"

# Set reference image to be used in setting extent, snap raster, cell size
refImage = arcpy.Raster("H:/NESSF_Sac/Landsat8/mask/Sac_WRS2_CVJV_extent.tif")

## Set environments to match the reference study area raster
# arcpy.env.extent = arcpy.Extent(-107.0, 38.0, -104.0, 40.0) # This method sets dimensions
arcpy.env.extent = refImage # This method references a file
arcpy.env.snapRaster = refImage
arcpy.env.cellSize = refImage

## Loop through each scene directory
## Search for scene directory list
dirlist = glob.glob("H:/NESSF_Sac/Landsat8/"+'04*')
print "Number of scenes: " +str(len(dirlist))
print dirlist

## Loop through each scene directory
for thisdir in dirlist:
    ## List the original cdr datasets available
    cdrlist = glob.glob(thisdir + "/raw_data/L*")
    print "Number of cdr files: " + str(len(cdrlist))
    ### Add log file?
    ###
    for cdr in cdrlist:
        ## Find band6
        b6list = glob.glob(cdr + "/*_sr_band6.img")
        print b6list
        
        ## Extract the SceneID string from band 6
        filename = b6list[0]
        print filename
        mysplit = filename.split("\\")
        lastname = mysplit[-1]
        print lastname
        mysplit2 = lastname.split("_")
        SceneID = mysplit2[0]
        
        ## Rescale band 6 (swir)
        arcpy.CheckOutExtension('Spatial')
        arcpy.env.overwriteOutput = True
        inB6 = arcpy.Raster(b6list[0])
        outB6 = inB6 * 0.0001

        ## Find band 3
        #b3list = glob.glob(cdr + "/*_sr_band3.img")
        #print b3list
        
        ## Rescale band 3
        #inB3 = arcpy.Raster(b3list[0])
        #outB3 = inB3 * 0.0001
              
        ## Calculate the MNDWI and save the output
        #mndwi = (outB3 - outB6) / (outB3 + outB6)
        #mndwi.save(thisdir + "\\output\\mndwi\\" + SceneID + "_sr_mndwi.img")

        ## Find the preprocessed ndvi layer
        ndvilist = glob.glob(cdr + "/*_ndvi.img")
        print ndvilist
        
        ## Rescale ndvi 
        inndvi = arcpy.Raster(ndvilist[0])
        outndvi = inndvi * 0.0001
                             
        ## Apply a threshold to band 6 to create a binary water map
        ## values =< 0.0690, 1
        ## values >= 0.0690, 0            
        inRaster = outB6
        reclassField = "Value"
        remap = arcpy.sa.RemapRange([[-30,0.0690, 1], [0.0690,30, 0]])
        reclassb6 = arcpy.sa.Reclassify(inRaster, "Value", remap, "NODATA")
        reclassb6.save(thisdir + "\\output\\scratch\\threshold.tif")

        ## Find the cfmask
        cflist = glob.glob(cdr + "/*_cfmask.img")
        print cflist
        
        ## Reclassify the cf mask to create a mask of good non-cloud pixels
        ## 0 = clear
        ## 1 = water
        ## 2 = shadow
        ## 3 = snow
        ## 4 = cloud
        ## 255 = fill
        cf = arcpy.Raster(cflist[0])
        #Reclassify non-cloud pixels to 1 and cloud pixels to NODATA
        remap2 = arcpy.sa.RemapRange([[0, 1, 1],[2,2,"NODATA"], [4,4,"NODATA"], [3,3,1]]) 
        reclasscf = arcpy.sa.Reclassify(cf,"Value", remap2, "NODATA")
        reclasscf.save(thisdir + "\\output\\noncloud\\" + SceneID + "_sr_noncloud.img")
        
        ## extract non-cloud pixels from band 6
        inRas = reclassb6
        mask = reclasscf
        outExtractCf = arcpy.sa.ExtractByMask(inRas, mask)
        outExtractCf.save(thisdir + "\\output\\scratch\\water_b6.tif")
        
        ## extract band 6 by the study area (combined CVJV plan area, elevation <70m, non-urban USDA CDL)
        inRas = outExtractCf
        mask = arcpy.Raster("H:/NESSF_Sac/Landsat8/mask/telemetry_mask_nonurban.tif")
        outExtract2 = arcpy.sa.ExtractByMask(inRas, mask)
        outExtract2.save(thisdir + "\\output\\scratch\\extract_nonurban.tif")
      
        ## Copy Raster and increase extent to match reference image
        #arcpy.env.extent = "555342.557462497 4208145 686832.557462497 4462575"
        arcpy.env.extent = refImage
        arcpy.env.snapRaster = refImage
        arcpy.CopyRaster_management(outExtract2, thisdir + "\\output\\masked_water_map\\" + SceneID + "_sr_water.img")

        ## Do the same cloud elimination procedure for the ndvi image
        ## extract non-cloud pixels from ndvi
        inRas = outndvi
        mask = reclasscf
        outExtractCf = arcpy.sa.ExtractByMask(inRas, mask)
        outExtractCf.save(thisdir + "\\output\\scratch\\ndvi_ncloud.tif")
        
        ## extract band 6 by the study area (combined CVJV plan area, elevation <70m, non-urban USDA CDL)
        inRas = outExtractCf
        mask = arcpy.Raster("H:/NESSF_Sac/Landsat8/mask/telemetry_mask_nonurban.tif")
        outExtract2 = arcpy.sa.ExtractByMask(inRas, mask)
        outExtract2.save(thisdir + "\\output\\scratch\\extract_ndvi_nonurban.tif")
      
        ## Copy Raster and increase extent to match reference image
        arcpy.env.extent = refImage
        arcpy.env.snapRaster = refImage
        arcpy.CopyRaster_management(outExtract2, thisdir + "\\output\\masked_ndvi\\" + SceneID + "_sr_ndvi.img")

        ## Clean up temporary files
        ## arcpy.Delete_management(tmpFC)
